if [ -x /usr/bin/scrollkeeper-update ]; then
  /usr/bin/scrollkeeper-update -p /var/lib/scrollkeeper 1> /dev/null 2> /dev/null
fi

if [ -x /usr/bin/update-mime-database ]; then
  /usr/bin/update-mime-database /usr/share/mime &> /dev/null
fi

if [ -x /usr/bin/update-desktop-database ]; then
  /usr/bin/update-desktop-database 1> /dev/null 2> /dev/null
fi

if [ -x /usr/bin/gtk-update-icon-cache ]; then
  /usr/bin/gtk-update-icon-cache  -f -q /usr/share/icons/hicolor 1> /dev/null 2> /dev/null
fi

( cd usr/share/icons/hicolor/24x24/mimetypes ; rm -rf application-x-cbr.png )
( cd usr/share/icons/hicolor/24x24/mimetypes ; ln -sf application-x-cbz.png application-x-cbr.png )
( cd usr/share/icons/hicolor/24x24/mimetypes ; rm -rf application-x-cbt.png )
( cd usr/share/icons/hicolor/24x24/mimetypes ; ln -sf application-x-cbz.png application-x-cbt.png )
( cd usr/share/icons/hicolor/16x16/mimetypes ; rm -rf application-x-cbr.png )
( cd usr/share/icons/hicolor/16x16/mimetypes ; ln -sf application-x-cbz.png application-x-cbr.png )
( cd usr/share/icons/hicolor/16x16/mimetypes ; rm -rf application-x-cbt.png )
( cd usr/share/icons/hicolor/16x16/mimetypes ; ln -sf application-x-cbz.png application-x-cbt.png )
( cd usr/share/icons/hicolor/22x22/mimetypes ; rm -rf application-x-cbr.png )
( cd usr/share/icons/hicolor/22x22/mimetypes ; ln -sf application-x-cbz.png application-x-cbr.png )
( cd usr/share/icons/hicolor/22x22/mimetypes ; rm -rf application-x-cbt.png )
( cd usr/share/icons/hicolor/22x22/mimetypes ; ln -sf application-x-cbz.png application-x-cbt.png )
( cd usr/share/icons/hicolor/32x32/mimetypes ; rm -rf application-x-cbr.png )
( cd usr/share/icons/hicolor/32x32/mimetypes ; ln -sf application-x-cbz.png application-x-cbr.png )
( cd usr/share/icons/hicolor/32x32/mimetypes ; rm -rf application-x-cbt.png )
( cd usr/share/icons/hicolor/32x32/mimetypes ; ln -sf application-x-cbz.png application-x-cbt.png )
( cd usr/share/icons/hicolor/48x48/mimetypes ; rm -rf application-x-cbr.png )
( cd usr/share/icons/hicolor/48x48/mimetypes ; ln -sf application-x-cbz.png application-x-cbr.png )
( cd usr/share/icons/hicolor/48x48/mimetypes ; rm -rf application-x-cbt.png )
( cd usr/share/icons/hicolor/48x48/mimetypes ; ln -sf application-x-cbz.png application-x-cbt.png )
( cd usr/bin ; rm -rf comix )
( cd usr/bin ; ln -sf ../share/comix/src/comix.py comix )
